@@ -5,13 +5,13 @@
 (defparameter *autoconf-prefix* "/usr/local")
 (defparameter *autoconf-exec_prefix* "/usr/local")
 (defparameter *autoconf-package* "maxima")
-(defparameter *autoconf-version* "5.13.0")
+(defparameter *autoconf-version* "5.16.1")
 (defparameter *autoconf-libdir* "/usr/local/lib")
 (defparameter *autoconf-libexecdir* "/usr/local/libexec")
 (defparameter *autoconf-datadir* "/usr/local/share")
 (defparameter *autoconf-infodir* "/usr/local/info")
-(defparameter *autoconf-host* "i386-unknown-openbsd4.4")
+(defparameter *autoconf-host* "x86_64-unknown-openbsd4.7")
 (defparameter *autoconf-win32* "false")
  
-(defparameter *maxima-source-root* "/usr/obj/i386/maxima-5.13.0/maxima-5.13.0")
+(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.16.1p2/maxima-5.16.1")
 (defparameter *maxima-default-layout-autotools* "true")
